--[[
Script to buy multiple pallets

Author:		Ifko[nator]
Date:		31.07.2021
Version:	1.0

History:	v1.0 @31.07.2021 - initial release in FS 19
]]

BuyMorePallets = {};

function BuyMorePallets:loadMap(name)
	g_configurationManager:addConfigurationType(
		"buyablePalletsAmount",
		RealisticSeederUtil.getL10nText("animals_foodMixQuantity"),
		nil,
		nil,
		nil,
		nil,
		ConfigurationUtil.SELECTOR_MULTIOPTION
	);

	Vehicle.load = Utils.overwrittenFunction(Vehicle.load, BuyMorePallets.vehicleLoad);
	Vehicle.saveToXMLFile = Utils.overwrittenFunction(Vehicle.saveToXMLFile, BuyMorePallets.vehicleSaveToXMLFile);
	
	ShopConfigScreen.setStoreItem = Utils.prependedFunction(ShopConfigScreen.setStoreItem, BuyMorePallets.shopConfigScreenSetStoreItem);
	ShopConfigScreen.setConfigPrice = Utils.overwrittenFunction(ShopConfigScreen.setConfigPrice, BuyMorePallets.shopConfigScreenSetConfigPrice);
	ShopConfigScreen.getConfigurationCostsAndChanges = Utils.overwrittenFunction(ShopConfigScreen.getConfigurationCostsAndChanges, BuyMorePallets.shopConfigScreenGetConfigurationCostsAndChanges);
	
	ShopController.update = Utils.prependedFunction(ShopController.update, BuyMorePallets.shopControllerUpdate);
	ShopController.onVehicleBought = Utils.appendedFunction(ShopController.onVehicleBought, BuyMorePallets.shopControllerOnVehicleBought);
	ShopController.onVehicleBuyFailed = Utils.appendedFunction(ShopController.onVehicleBuyFailed, BuyMorePallets.shopControllerOnVehicleBuyFailed);
end;

function BuyMorePallets:vehicleLoad(superFunc, vehicleData, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments)
	local storeItem = g_storeManager:getItemByXMLFilename(vehicleData.filename);
	
	if storeItem ~= nil and RealisticSeederUtil.getIsValidStoreCategoryName(storeItem.categoryName) then
		if storeItem.configurations ~= nil then
			if storeItem.configurations.buyablePalletsAmount == nil then
				local configurationItems = {};

				for palletNumber = 1, RealisticSeederUtil.maxPalletsAmount do
					StoreItemUtil.addConfigurationItem(
						configurationItems,
						tostring(palletNumber),
						nil,
						0,
						0,
						false
					);
				end;

				storeItem.configurations.buyablePalletsAmount = configurationItems;
			end;
		end;
	end;

	return superFunc(self, vehicleData, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments);
end;

function BuyMorePallets:vehicleSaveToXMLFile(superFunc, xmlFile, key, usedModNames)
	if self.configurations.buyablePalletsAmount ~= nil then
		self.configurations.buyablePalletsAmount = nil;
		self.boughtConfigurations.buyablePalletsAmount = nil;
	end;

	return superFunc(self, xmlFile, key, usedModNames);
end;

function BuyMorePallets:shopConfigScreenSetStoreItem(storeItem, vehicle, configBasePrice)
	if RealisticSeederUtil.getIsValidStoreCategoryName(storeItem.categoryName) then
		if storeItem.configurations.buyablePalletsAmount == nil then
			configurationItems = {};

			local xmlFile = loadXMLFile("vehicle", storeItem.xmlFilename);
			local text = RealisticSeederUtil.getL10nText(getXMLString(xmlFile, "vehicle.base.typeDesc"));
			
			for palletNumber = 1, 10 do
				if palletNumber > 1 then
					text = RealisticSeederUtil.getMultipleText(getXMLString(xmlFile, "vehicle.base.typeDesc"));
				end;
				
				StoreItemUtil.addConfigurationItem(
					configurationItems,
					tostring(palletNumber).. " " .. text,
					nil,
					0,
					0,
					false
				);
			end;
			
			storeItem.configurations.buyablePalletsAmount = configurationItems;
			
			delete(xmlFile);
		end;
	end;
end;

function BuyMorePallets:shopConfigScreenSetConfigPrice(superFunc, configName, configIndex, priceTextElement, vehicle)
	local storeItem = self.storeItem;
	
	if RealisticSeederUtil.getIsValidStoreCategoryName(storeItem.categoryName) then
		local price = (configIndex - 1) * storeItem.price;

		priceTextElement:setText("+" .. self.l10n:formatMoney(price));
		priceTextElement:setVisible(true);
	else
		return superFunc(self, configName, configIndex, priceTextElement, vehicle);
	end;
end;

function BuyMorePallets:shopConfigScreenGetConfigurationCostsAndChanges(superFunc, storeItem, vehicle)
	local basePrice = 0;
	local upgradePrice = 0;
	local hasChanges = false;

	if vehicle ~= nil then
		for name, id in pairs(self.configurations) do
			if vehicle.configurations[name] ~= id then
				hasChanges = true;

				if not ConfigurationUtil.hasBoughtConfiguration(self.vehicle, name, id) then
					local configs = storeItem.configurations[name];
					local price = math.max(configs[id].price - configs[self.vehicle.configurations[name]].price, 0);

					upgradePrice = upgradePrice + price;
				end;
			end;
		end;
	elseif storeItem ~= nil then
		hasChanges = true;
		basePrice, upgradePrice = self.economyManager:getBuyPrice(storeItem, self.configurations);
		basePrice = basePrice - upgradePrice;

		if RealisticSeederUtil.getIsValidStoreCategoryName(storeItem.categoryName) then
			if self.configurations.buyablePalletsAmount ~= nil then
				upgradePrice = (self.configurations.buyablePalletsAmount - 1) * basePrice;
			end;
		end;
	end;

	return basePrice, upgradePrice, hasChanges;
end;

function BuyMorePallets:shopControllerUpdate(dt)
	if self.buyVehicleNow == 2 then
		if self.buyItemConfigurations.buyablePalletsAmount ~= nil then
			BuyMorePallets.buyablePalletsAmount = self.buyItemConfigurations.buyablePalletsAmount;
			BuyMorePallets.palletsAmountBought = 0;
			BuyMorePallets.palletsAmountFailed = 0;
		end;
	end;
end;

function BuyMorePallets:shopControllerOnVehicleBought(leaseVehicle, price)
	if self.buyItemConfigurations.buyablePalletsAmount ~= nil then
		BuyMorePallets.palletsAmountBought = BuyMorePallets.palletsAmountBought + 1;

		if BuyMorePallets.palletsAmountBought < BuyMorePallets.buyablePalletsAmount then
			self.client:getServerConnection():sendEvent(BuyVehicleEvent:new(self.buyItemFilename, self.buyItemIsOutsideBuy, self.buyItemConfigurations, self.buyItemIsLeasing, self.playerFarmId));
		end;
	end;
end;

function BuyMorePallets:shopControllerOnVehicleBuyFailed(leaseVehicle, errorCode)
	if self.buyItemConfigurations.buyablePalletsAmount ~= nil then
		BuyMorePallets.palletsAmountFailed = BuyMorePallets.palletsAmountFailed + 1;
		
		if errorCode == BuyVehicleEvent.STATE_NO_SPACE then
			g_gui:closeAllDialogs();

			g_gui:showInfoDialog({
				text = string.format(RealisticSeederUtil.getL10nText("warning_messageNoSpace"), BuyMorePallets.palletsAmountBought, BuyMorePallets.buyablePalletsAmount),
				callback = self.onBoughtCallback,
				target = self
			});
		end;
	end;
end;

function BuyMorePallets:update(dt)end;
function BuyMorePallets:deleteMap()end;
function BuyMorePallets:mouseEvent(posX, posY, isDown, isUp, button)end;
function BuyMorePallets:keyEvent(unicode, sym, modifier, isDown)end;
function BuyMorePallets:draw()end;

addModEventListener(BuyMorePallets);