--[[
LanguageChecker.lua

Author: 	Ifko[nator]
Date:		13.06.2021
Version:	1.4

History:	v1.0 @14.05.2020 - initial implemation in FS 19
			-------------------------------------------------------------------------------------
			v1.1 @11.07.2020 - added support for all ingame languages
			-------------------------------------------------------------------------------------
			v1.2 @14.02.2021 - reworkt load function
			-------------------------------------------------------------------------------------
			v1.3 @15.02.2021 - added hook function for the load function of the BuyingStation.lua
			-------------------------------------------------------------------------------------
			v1.4 @13.06.2021 - minior adjustments
]]

LanguageChecker = {};
LanguageChecker.currentModName = g_currentModName;

function LanguageChecker.prerequisitesPresent(specializations)
	return true;
end;

function LanguageChecker.registerEventListeners(vehicleType)
	local functionNames = {
		"onPostLoad"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, LanguageChecker);
	end;
end;

function LanguageChecker:onPostLoad(savegame)
	if RealisticSeederUtil == nil then
		return;
	end;
	
	if RealisticSeederUtil.languageShort ~= "de" then
		LanguageChecker.debugPriority = RealisticSeederUtil.getDebugPriority(self.xmlFile, "vehicle.languageChecker#debugPriority");

		local materialNumber = 0;

		while true do
			local materialKey = "vehicle.languageChecker.material(" .. tostring(materialNumber) .. ")";

			if not hasXMLProperty(self.xmlFile, materialKey) then
				break;
			end;

			local translateNode = I3DUtil.indexToObject(self.components, Utils.getNoNil(getXMLString(self.xmlFile, materialKey .. "." .. RealisticSeederUtil.languageShort .. "#node"), getXMLString(self.xmlFile, materialKey .. ".en#node")), self.i3dMappings);
			local materialNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, materialKey .. "#node"), self.i3dMappings);
			local visibillityMaterialNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, materialKey .. "#visibillityNode"), self.i3dMappings);
			local languageShort = getXMLString(self.xmlFile, materialKey .. "#languageShort");

			if materialNode ~= nil and visibillityMaterialNode ~= nil and languageShort ~= nil then
				if RealisticSeederUtil.languageShort == languageShort then
					setVisibillity(visibillityMaterialNode, true);
					setVisibillity(materialNode, false);
				end;
			end;

			if materialNode ~= nil and translateNode ~= nil then
				RealisticSeederUtil.changeMaterial(self, materialNode, translateNode);
				
				RealisticSeederUtil.printDebug("Changed material for " .. getName(materialNode) .. " successfully.", LanguageChecker.debugPriority, true, "LanguageChecker");
			end;
			
			materialNumber = materialNumber + 1;
		end;
    end;
end;

function LanguageChecker:loadBuyingStation(superFunc, xmlFilename, x, y, z, rx, ry, rz, initRandom)
	if not BuyingStationPlaceable:superClass().load(self, xmlFilename, x, y, z, rx, ry, rz, initRandom) then
		return false;
	end;

	local xmlFile = loadXMLFile("TempXML", xmlFilename);

	self.buyingStation = BuyingStation:new(self.isServer, self.isClient);

	self.buyingStation:load(self.nodeId, xmlFile, "placeable.buyingStation", self.customEnvironment);

	self.buyingStation.owningPlaceable = self;

	if RealisticSeederUtil ~= nil and RealisticSeederUtil.languageShort ~= "de" then
		local debugPriority = RealisticSeederUtil.getDebugPriority(xmlFile, "placeable.buyingStation.languageChecker#debugPriority");
		local materialNumber = 0;

		while true do
			local materialKey = "placeable.buyingStation.languageChecker.material(" .. tostring(materialNumber) .. ")";

			if not hasXMLProperty(xmlFile, materialKey) then
				break;
			end;

			local translateNode = I3DUtil.indexToObject(self.nodeId, Utils.getNoNil(getXMLString(xmlFile, materialKey .. "." .. RealisticSeederUtil.languageShort .. "#node"), getXMLString(xmlFile, materialKey .. ".en#node")));
			local materialNode = I3DUtil.indexToObject(self.nodeId, getXMLString(xmlFile, materialKey .. "#node"));

			if materialNode ~= nil and translateNode ~= nil then
				RealisticSeederUtil.changeMaterial(self, materialNode, translateNode);

				RealisticSeederUtil.printDebug("Changed material for " .. getName(materialNode) .. " successfully.", debugPriority, true, "LanguageChecker");
			end;

			materialNumber = materialNumber + 1;
		end;
	end;

	delete(xmlFile);

	return true;
end;

BuyingStationPlaceable.load = Utils.overwrittenFunction(BuyingStationPlaceable.load, LanguageChecker.loadBuyingStation);