--[[
RegisterFillTypes.lua
	
Author: 	Ifko[nator]
Date:		13.06.2021
Version:	1.4

History:	v1.0 @14.05.2020 - initial implementation in FS 19
			------------------------------------------------------------------------------------------------------------
			v1.2 @01.08.2020 - fill sounds will now also loaded for the new fill types
			------------------------------------------------------------------------------------------------------------
			v1.3 @27.08.2020 - reworkt function to load fill types and fill sounds, so it is not mess up with other mods
			------------------------------------------------------------------------------------------------------------
			v1.4 @13.06.2021 - minior adjustments
]]

RegisterFillTypes = {};

function RegisterFillTypes:loadedMap(mission, node)
	if RealisticSeederUtil == nil then
        return;
	end;
	
	if fileExists(RealisticSeederUtil.fillTypesPath) then
		g_fillTypeManager:loadFillTypes(RealisticSeederUtil.fillTypesXML, RealisticSeederUtil.currentModName, RealisticSeederUtil.currentModDirectory, true);
	else
		RealisticSeederUtil.printError("Can't load fillTypesXML file (path: " .. RealisticSeederUtil.fillTypesPath .. ")! Aborting register fill types!", false, false, "RegisterFillTypes");
	end;
end;

function RegisterFillTypes:addFillType(superFunc, name, title, showOnPriceTable, pricePerLiter, massPerLiter, maxPhysicalSurfaceAngle, hudOverlayFilename, hudOverlayFilenameSmall, baseDirectory, customEnv, fillPlaneColors, palletFilename, isBaseType)
	if RealisticSeederUtil == nil then
		return;
	end;
	
	if name == nil or name == "" or name:find("[^%w_.]") then
		RealisticSeederUtil.printError("'" .. tostring(name) .. "' is not a valid name for a fillType. Ignoring fillType!", true, false, "RegisterFillTypes");

		return nil;
	end;

	name = name:upper();

	if isBaseType and self.nameToFillType[name] ~= nil then
		RealisticSeederUtil.printError("FillType '" .. tostring(name) .. "' already exists. Ignoring fillType!", true, false, "RegisterFillTypes");

		return nil;
	end;

	local fillType = self.nameToFillType[name];

	if fillType == nil then
		--## no limit for us :D
		--if #self.fillTypes > 256 then
		--	RealisticSeederUtil.printError("FillTypeManager.addFillType too many fill types. Only 256 fill types are supported", false, false, "RegisterFillTypes");

		--	return;
		--end;

		fillType = {
			name = name,
			index = #self.fillTypes + 1,
			title = g_i18n:convertText(title, customEnv)
		};

		self.nameToFillType[name] = fillType;
		self.nameToIndex[name] = fillType.index;
		self.indexToName[fillType.index] = name;

		table.insert(self.fillTypes, fillType);
	end;

	fillType.showOnPriceTable = Utils.getNoNil(showOnPriceTable, Utils.getNoNil(fillType.showOnPriceTable, false));
	fillType.pricePerLiter = Utils.getNoNil(pricePerLiter, Utils.getNoNil(fillType.pricePerLiter, 0));
	fillType.massPerLiter = Utils.getNoNil(massPerLiter, Utils.getNoNil(fillType.massPerLiter, 0.0001)) * FillTypeManager.MASS_SCALE;
	fillType.maxPhysicalSurfaceAngle = Utils.getNoNilRad(maxPhysicalSurfaceAngle, Utils.getNoNil(fillType.maxPhysicalSurfaceAngle, math.rad(30)));
	fillType.hudOverlayFilename = Utils.getFilename(hudOverlayFilename, baseDirectory) or fillType.hudOverlayFilename;
	fillType.hudOverlayFilenameSmall = Utils.getFilename(hudOverlayFilenameSmall, baseDirectory) or fillType.hudOverlayFilenameSmall;

	if fillType.index ~= FillType.UNKNOWN then
		if fillType.hudOverlayFilename == nil or fillType.hudOverlayFilename == "" then
			RealisticSeederUtil.printError("FillType '" .. name .. "' has no valid image assigned!", true, false, "RegisterFillTypes");
		end;

		if fillType.hudOverlayFilenameSmall == nil or fillType.hudOverlayFilenameSmall == "" then
			RealisticSeederUtil.printError("FillType '" .. name .. "' has no valid small image assigned!", true, false, "RegisterFillTypes");
		end;
	end;

	if palletFilename ~= nil then
		fillType.palletFilename = Utils.getFilename(palletFilename, baseDirectory) or fillType.palletFilename;
	end;

	fillType.previousHourPrice = fillType.pricePerLiter;
	fillType.startPricePerLiter = fillType.pricePerLiter;
	fillType.totalAmount = FillTypeManager.FILLTYPE_START_TOTAL_AMOUNT;
	fillType.fillPlaneColors = {};

	if fillPlaneColors ~= nil then
		fillType.fillPlaneColors[1] = fillPlaneColors[1] or fillType.fillPlaneColors[1];
		fillType.fillPlaneColors[2] = fillPlaneColors[2] or fillType.fillPlaneColors[2];
		fillType.fillPlaneColors[3] = fillPlaneColors[3] or fillType.fillPlaneColors[3];
	else
		fillType.fillPlaneColors[1] = fillType.fillPlaneColors[1] or 1;
		fillType.fillPlaneColors[2] = fillType.fillPlaneColors[2] or 1;
		fillType.fillPlaneColors[3] = fillType.fillPlaneColors[3] or 1;
	end;

	return fillType;
end;

function RegisterFillTypes:loadFillTypes(superFunc, xmlFile, missionInfo, baseDirectory, isBaseType)
	self:addFillType("UNKNOWN", "Unknown", false, 0, 0, 0, "", "", baseDirectory, nil, nil);

	local fillTypeNumber = 0;

	while true do
		local fillTypeKey = string.format("map.fillTypes.fillType(%d)", fillTypeNumber);

		if not hasXMLProperty(xmlFile, fillTypeKey) then
			break;
		end;

		local name = getXMLString(xmlFile, fillTypeKey .. "#name");
		local title = getXMLString(xmlFile, fillTypeKey .. "#title");
		local showOnPriceTable = getXMLBool(xmlFile, fillTypeKey .. "#showOnPriceTable");
		local pricePerLiter = getXMLFloat(xmlFile, fillTypeKey .. "#pricePerLiter");
		local fillPlaneColorsString = getXMLString(xmlFile, fillTypeKey .. "#fillPlaneColors");
		local fillPlaneColors = {1, 1, 1};

		if fillPlaneColorsString ~= nil then
			fillPlaneColors = StringUtil.getVectorNFromString(fillPlaneColorsString, 3);
		end;

		local massPerLiter = getXMLFloat(xmlFile, fillTypeKey .. ".physics#massPerLiter") / 1000;
		local maxPhysicalSurfaceAngle = getXMLFloat(xmlFile, fillTypeKey .. ".physics#maxPhysicalSurfaceAngle");
		local hudFilename = getXMLString(xmlFile, fillTypeKey .. ".image#hud");
		local hudSmallFilename = getXMLString(xmlFile, fillTypeKey .. ".image#hudSmall");
		local palletFilename = getXMLString(xmlFile, fillTypeKey .. ".pallet#filename");
		local customEnv = nil;

		if missionInfo ~= nil then
			if missionInfo.customEnvironment ~= nil then	
				customEnv = missionInfo.customEnvironment;
			else
				customEnv = missionInfo;
			end;
		end;

		self:addFillType(name, title, showOnPriceTable, pricePerLiter, massPerLiter, maxPhysicalSurfaceAngle, hudFilename, hudSmallFilename, baseDirectory, customEnv, fillPlaneColors, palletFilename, isBaseType or false);

		fillTypeNumber = fillTypeNumber + 1;
	end;

	local fillTypeCategoryNumber = 0;

	while true do
		local fillTypeCategoryKey = string.format("map.fillTypeCategories.fillTypeCategory(%d)", fillTypeCategoryNumber);

		if not hasXMLProperty(xmlFile, fillTypeCategoryKey) then
			break;
		end;

		local name = getXMLString(xmlFile, fillTypeCategoryKey .. "#name");
		local fillTypesStr = getXMLString(xmlFile, fillTypeCategoryKey);
		local fillTypeCategoryIndex = self:addFillTypeCategory(name, isBaseType);

		if fillTypeCategoryIndex ~= nil then
			local fillTypeNames = StringUtil.splitString(" ", fillTypesStr);

			for _, fillTypeName in ipairs(fillTypeNames) do
				local fillType = self:getFillTypeByName(fillTypeName);

				if fillType ~= nil then
					if not self:addFillTypeToCategory(fillType.index, fillTypeCategoryIndex) then
						print("Warning: Could not add fillType '" .. tostring(fillTypeName) .. "' to fillTypeCategory '" .. tostring(name) .. "'!");
					end;
				else
					print("Warning: FillType '" .. tostring(fillTypeName) .. "' not defined in fillTypeCategory '" .. tostring(name) .. "'!");
				end;
			end;
		end;

		fillTypeCategoryNumber = fillTypeCategoryNumber + 1;
	end;

	local fillTypeConverterNumber = 0;

	while true do
		local fillTypeConverterKey = string.format("map.fillTypeConverters.fillTypeConverter(%d)", fillTypeConverterNumber);

		if not hasXMLProperty(xmlFile, fillTypeConverterKey) then
			break;
		end;

		local name = getXMLString(xmlFile, fillTypeConverterKey .. "#name");
		local converter = self:addFillTypeConverter(name, isBaseType);

		if converter ~= nil then
			local converterNumber = 0;

			while true do
				local converterKey = string.format("%s.converter(%d)", fillTypeConverterKey, converterNumber);

				if not hasXMLProperty(xmlFile, converterKey) then
					break;
				end;

				local from = getXMLString(xmlFile, converterKey .. "#from");
				local to = getXMLString(xmlFile, converterKey .. "#to");
				local factor = getXMLFloat(xmlFile, converterKey .. "#factor");
				local sourceFillType = g_fillTypeManager:getFillTypeByName(from);
				local targetFillType = g_fillTypeManager:getFillTypeByName(to);

				if sourceFillType ~= nil and targetFillType ~= nil and factor ~= nil then
					self:addFillTypeConversion(converter, sourceFillType.index, targetFillType.index, factor);
				end;

				converterNumber = converterNumber + 1
			end;
		end;

		fillTypeConverterNumber = fillTypeConverterNumber + 1;
	end;

	local fillTypeSoundNumber = 0;

	while true do
		local fillTypeSoundKey = string.format("map.fillTypeSounds.fillTypeSound(%d)", fillTypeSoundNumber);

		if not hasXMLProperty(xmlFile, fillTypeSoundKey) then
			break;
		end;

		local sample = g_soundManager:loadSampleFromXML(xmlFile, fillTypeSoundKey, "sound", baseDirectory, getRootNode(), 0, AudioGroup.VEHICLE, nil, nil);

		if sample ~= nil then
			local entry = {
				sample = sample,
				fillTypes = {}
			};

			local fillTypesStr = getXMLString(xmlFile, fillTypeSoundKey .. "#fillTypes");

			if fillTypesStr ~= nil then
				local fillTypeNames = StringUtil.splitString(" ", fillTypesStr);

				for _, fillTypeName in ipairs(fillTypeNames) do
					local fillType = self:getFillTypeIndexByName(fillTypeName);

					if fillType ~= nil then
						table.insert(entry.fillTypes, fillType);

						self.fillTypeToSample[fillType] = sample;
					else
						g_logManager:warning("Unable to load fill type '%s' for fillTypeSound '%s'", fillTypeName, key);
					end;
				end;
			end;

			if getXMLBool(xmlFile, fillTypeSoundKey .. "#isDefault") then
				for fillType, _ in ipairs(self.fillTypes) do
					if self.fillTypeToSample[fillType] == nil then
						self.fillTypeToSample[fillType] = sample;
					end;
				end;
			end;

			table.insert(self.fillTypeSamples, entry);
		end;

		fillTypeSoundNumber = fillTypeSoundNumber + 1;
	end;

	return true;
end;

FillTypeManager.addFillType = Utils.overwrittenFunction(FillTypeManager.addFillType, RegisterFillTypes.addFillType);
FillTypeManager.loadFillTypes = Utils.overwrittenFunction(FillTypeManager.loadFillTypes, RegisterFillTypes.loadFillTypes);

FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, RegisterFillTypes.loadedMap);

addModEventListener(RegisterFillTypes);