--[[
StoreManagerExtension.lua
	
Author: 	Ifko[nator]
Date:		13.06.2021
Version:	1.9

History:	v1.0 @04.07.2020 - initial implementation in FS 19
            -------------------------------------------------------------------------------------------------------------------------------
            v1.1 @11.07.2020 - added support for all ingame languages
            -------------------------------------------------------------------------------------------------------------------------------
            v1.2 @11.08.2020 - default seed is now more expensive
            -------------------------------------------------------------------------------------------------------------------------------
            v1.3 @27.08.2020 - reworkt function to load fill types, so it is not mess up with other mods
            -------------------------------------------------------------------------------------------------------------------------------
            v1.4 @29.08.2020 - store images for english and french players will now load correctly
            -------------------------------------------------------------------------------------------------------------------------------
            v1.5 @28.10.2020 - some script optimation
            -------------------------------------------------------------------------------------------------------------------------------
            v1.6 @10.02.2021 - removed a print call
            -------------------------------------------------------------------------------------------------------------------------------
            v1.7 @14.02.2021 - reworkt function to remove seed pallets/bigBags from shop wich fill type is not supportet by the current map
            -------------------------------------------------------------------------------------------------------------------------------
            v1.8 @05.06.2021 - some script optimation
            -------------------------------------------------------------------------------------------------------------------------------
            v1.9 @13.06.2021 - minior adjustments
]]

StoreManagerExtension = {};

function StoreManagerExtension:onStartMission()
    if RealisticSeederUtil == nil then
		return;
	end;
    
    StoreManagerExtension.debugPriority = RealisticSeederUtil.getDebugPriority(RealisticSeederUtil.modDesc, "modDesc.realisticSeeder#debugPriority");

    if RealisticSeederUtil.languageShort ~= "de" then
        local newStoreImages = {};
        local storeImageNumber = 0;

        while true do
            local storeImageKey = "modDesc.storeItems.storeImages.storeImage_" .. RealisticSeederUtil.languageShort .. "(" .. tostring(storeImageNumber) .. ")";

            if not hasXMLProperty(RealisticSeederUtil.modDesc, storeImageKey) then
                storeImageKey = "modDesc.storeItems.storeImages.storeImage_en(" .. tostring(storeImageNumber) .. ")";

                if not hasXMLProperty(RealisticSeederUtil.modDesc, storeImageKey) then
                    break;
                end;
            end;

            local storeImage_de = Utils.getNoNil(getXMLString(RealisticSeederUtil.modDesc, "modDesc.storeItems.storeImages.storeImage_de(" .. tostring(storeImageNumber) .. ")#file"), "");
            local storeImage_currentLanguageShort = Utils.getNoNil(getXMLString(RealisticSeederUtil.modDesc, storeImageKey .. "#file"), "");

            local storeImage = {};

            if storeImage_de ~= "" and storeImage_currentLanguageShort ~= "" then
                storeImage.de = storeImage_de;
                storeImage.currentLanguageShort = storeImage_currentLanguageShort;

                table.insert(newStoreImages, storeImage);
            end;

            storeImageNumber = storeImageNumber + 1;
        end;

        for _, storeItem in pairs(g_storeManager.items) do
            if string.find(storeItem.imageFilename, LanguageChecker.currentModName) then
                for _, storeImage in pairs(newStoreImages) do 
                    if string.find(storeItem.imageFilename, storeImage.de) then 
                        --## swap store image
                        storeItem.imageFilename = Utils.getFilename(storeImage.currentLanguageShort, RealisticSeederUtil.currentModDirectory);

                        RealisticSeederUtil.printDebug("Swaped store image  " .. storeImage.de .. " with " .. storeItem.imageFilename .. ".", StoreManagerExtension.debugPriority, true, "StoreManagerExtension");
                    end;
                end;
            end;
        end;
    end;

    for _, storeItem in pairs(g_storeManager.items) do
        if string.find(storeItem.xmlFilename, "data/objects/bigBagContainer/bigBagContainerSeeds.xml") or string.find(storeItem.xmlFilename, "data/objects/pallets/pioneer/pioneerPallet.xml") then
            --## make default seed more expensive
            storeItem.price = storeItem.price * 1.5;
        end;
    end;

    local fillTypes = RealisticSeederUtil.getSeedFillTypes();
    
    if fillTypes ~= nil then
        for _, fillType in pairs(fillTypes) do
            --## check if selected fruit is available in current map
            for _, storeItem in pairs(g_storeManager.items) do
                if not RealisticSeederUtil.getCanFruitBePlanted(fillType) and string.find(string.upper(storeItem.xmlFilename), g_fillTypeManager:getFillTypeByIndex(fillType).name) then
                    storeItem.showInStore = false;

                    RealisticSeederUtil.printDebug("Removed store item " .. storeItem.xmlFilename .. " from the shop due to not supportet fillType (" .. g_fillTypeManager:getFillTypeByIndex(fillType).title .. ")!", StoreManagerExtension.debugPriority, true, "StoreManagerExtension");
                end;
            end;
        end;
    end;

    if g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename ~= nil then
        if g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename == "data/objects/bigBagContainer/bigBagContainerSeeds.xml" then
           
            --## change default seed big bag with pioneer pallet for unloading
            g_fillTypeManager:getFillTypeByName("SEEDS").palletFilename = "data/objects/pallets/pioneer/pioneerPallet.xml";
        end;
    end;
end;

FSBaseMission.onStartMission = Utils.prependedFunction(FSBaseMission.onStartMission, StoreManagerExtension.onStartMission);

addModEventListener(StoreManagerExtension);